
package org.mule.module.intacct.processors;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.WildcardType;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import org.mule.api.MessagingException;
import org.mule.api.MuleContext;
import org.mule.api.MuleEvent;
import org.mule.api.MuleException;
import org.mule.api.MuleMessage;
import org.mule.api.construct.FlowConstruct;
import org.mule.api.construct.FlowConstructAware;
import org.mule.api.context.MuleContextAware;
import org.mule.api.expression.ExpressionManager;
import org.mule.api.lifecycle.Disposable;
import org.mule.api.lifecycle.Initialisable;
import org.mule.api.lifecycle.InitialisationException;
import org.mule.api.lifecycle.Startable;
import org.mule.api.lifecycle.Stoppable;
import org.mule.api.processor.MessageProcessor;
import org.mule.api.registry.RegistrationException;
import org.mule.api.transformer.DataType;
import org.mule.api.transformer.Transformer;
import org.mule.api.transformer.TransformerException;
import org.mule.config.i18n.CoreMessages;
import org.mule.config.i18n.MessageFactory;
import org.mule.module.intacct.IntacctCloudConnector;
import org.mule.module.intacct.adapters.IntacctCloudConnectorLifecycleAdapter;
import org.mule.transformer.TransformerTemplate;
import org.mule.transformer.types.DataTypeFactory;
import org.mule.transport.NullPayload;
import org.mule.util.TemplateParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * CreateSOTransactionMessageProcessor invokes the {@link org.mule.module.intacct.IntacctCloudConnector#createSOTransaction(java.lang.String, java.lang.String, java.util.Map, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.util.Map, java.lang.String, java.lang.String, org.mule.module.intacct.ContactType, java.util.List, org.mule.module.intacct.ContactType, java.util.List, java.lang.String, java.lang.String, java.lang.String, org.mule.module.intacct.ExchType, java.util.List, java.lang.String, java.util.List, java.util.List, java.util.List)} method in {@link IntacctCloudConnector }. For each argument there is a field in this processor to match it.  Before invoking the actual method the processor will evaluate and transform where possible to the expected argument type.
 * 
 */
public class CreateSOTransactionMessageProcessor
    implements FlowConstructAware, MuleContextAware, Disposable, Initialisable, Startable, Stoppable, MessageProcessor
{

    private Object functionControlId;
    private String _functionControlIdType;
    private Object transactionType;
    private String _transactionTypeType;
    private Object dateCreated;
    private Map<String, Object> _dateCreatedType;
    private Object createdFrom;
    private String _createdFromType;
    private Object customerId;
    private String _customerIdType;
    private Object documentNo;
    private String _documentNoType;
    private Object referenceNo;
    private String _referenceNoType;
    private Object termName;
    private String _termNameType;
    private Object dateDue;
    private Map<String, Object> _dateDueType;
    private Object message;
    private String _messageType;
    private Object shippingMethod;
    private String _shippingMethodType;
    private Object billToContactType;
    private org.mule.module.intacct.ContactType _billToContactTypeType;
    private Object billToContacts;
    private List<Map<String, Object>> _billToContactsType;
    private Object shipToContactType;
    private org.mule.module.intacct.ContactType _shipToContactTypeType;
    private Object shipToContacts;
    private List<Map<String, Object>> _shipToContactsType;
    private Object externalId;
    private String _externalIdType;
    private Object basecurr;
    private String _basecurrType;
    private Object currency;
    private String _currencyType;
    private Object exchType;
    private org.mule.module.intacct.ExchType _exchTypeType;
    private Object exchRateDatesOrExchRateTypesOrExchRates;
    private List<Map<String, Object>> _exchRateDatesOrExchRateTypesOrExchRatesType;
    private Object vsoePriceList;
    private String _vsoePriceListType;
    private Object customFields;
    private List<Map<String, Object>> _customFieldsType;
    private Object sOTransItems;
    private List<Map<String, Object>> _sOTransItemsType;
    private Object subTotals;
    private List<Map<String, Object>> _subTotalsType;
    private static Logger logger = LoggerFactory.getLogger(CreateSOTransactionMessageProcessor.class);
    /**
     * Module object
     * 
     */
    private Object moduleObject;
    /**
     * Mule Context
     * 
     */
    private MuleContext muleContext;
    /**
     * Mule Expression Manager
     * 
     */
    private ExpressionManager expressionManager;
    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;
    /**
     * Flow construct
     * 
     */
    private FlowConstruct flowConstruct;
    /**
     * Variable used to track how many retries we have attempted on this message processor
     * 
     */
    private AtomicInteger retryCount;
    /**
     * Maximum number of retries that can be attempted.
     * 
     */
    private int retryMax;

    /**
     * Obtains the expression manager from the Mule context and initialises the connector. If a target object  has not been set already it will search the Mule registry for a default one.
     * 
     * @throws InitialisationException
     */
    public void initialise()
        throws InitialisationException
    {
        retryCount = new AtomicInteger();
        expressionManager = muleContext.getExpressionManager();
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
        if (moduleObject == null) {
            try {
                moduleObject = muleContext.getRegistry().lookupObject(IntacctCloudConnectorLifecycleAdapter.class);
                if (moduleObject == null) {
                    throw new InitialisationException(MessageFactory.createStaticMessage("Cannot find object"), this);
                }
            } catch (RegistrationException e) {
                throw new InitialisationException(CoreMessages.initialisationFailure("org.mule.module.intacct.adapters.IntacctCloudConnectorLifecycleAdapter"), e, this);
            }
        }
        if (moduleObject instanceof String) {
            moduleObject = muleContext.getRegistry().lookupObject(((String) moduleObject));
            if (moduleObject == null) {
                throw new InitialisationException(MessageFactory.createStaticMessage("Cannot find object by config name"), this);
            }
        }
    }

    public void start()
        throws MuleException
    {
    }

    public void stop()
        throws MuleException
    {
    }

    public void dispose() {
    }

    /**
     * Set the Mule context
     * 
     * @param context Mule context to set
     */
    public void setMuleContext(MuleContext context) {
        this.muleContext = context;
    }

    /**
     * Sets flow construct
     * 
     * @param flowConstruct Flow construct to set
     */
    public void setFlowConstruct(FlowConstruct flowConstruct) {
        this.flowConstruct = flowConstruct;
    }

    /**
     * Sets the instance of the object under which the processor will execute
     * 
     * @param moduleObject Instace of the module
     */
    public void setModuleObject(Object moduleObject) {
        this.moduleObject = moduleObject;
    }

    /**
     * Sets retryMax
     * 
     * @param value Value to set
     */
    public void setRetryMax(int value) {
        this.retryMax = value;
    }

    /**
     * Sets functionControlId
     * 
     * @param value Value to set
     */
    public void setFunctionControlId(Object value) {
        this.functionControlId = value;
    }

    /**
     * Sets termName
     * 
     * @param value Value to set
     */
    public void setTermName(Object value) {
        this.termName = value;
    }

    /**
     * Sets customFields
     * 
     * @param value Value to set
     */
    public void setCustomFields(Object value) {
        this.customFields = value;
    }

    /**
     * Sets basecurr
     * 
     * @param value Value to set
     */
    public void setBasecurr(Object value) {
        this.basecurr = value;
    }

    /**
     * Sets shipToContacts
     * 
     * @param value Value to set
     */
    public void setShipToContacts(Object value) {
        this.shipToContacts = value;
    }

    /**
     * Sets billToContacts
     * 
     * @param value Value to set
     */
    public void setBillToContacts(Object value) {
        this.billToContacts = value;
    }

    /**
     * Sets dateDue
     * 
     * @param value Value to set
     */
    public void setDateDue(Object value) {
        this.dateDue = value;
    }

    /**
     * Sets subTotals
     * 
     * @param value Value to set
     */
    public void setSubTotals(Object value) {
        this.subTotals = value;
    }

    /**
     * Sets exchRateDatesOrExchRateTypesOrExchRates
     * 
     * @param value Value to set
     */
    public void setExchRateDatesOrExchRateTypesOrExchRates(Object value) {
        this.exchRateDatesOrExchRateTypesOrExchRates = value;
    }

    /**
     * Sets shippingMethod
     * 
     * @param value Value to set
     */
    public void setShippingMethod(Object value) {
        this.shippingMethod = value;
    }

    /**
     * Sets shipToContactType
     * 
     * @param value Value to set
     */
    public void setShipToContactType(Object value) {
        this.shipToContactType = value;
    }

    /**
     * Sets externalId
     * 
     * @param value Value to set
     */
    public void setExternalId(Object value) {
        this.externalId = value;
    }

    /**
     * Sets sOTransItems
     * 
     * @param value Value to set
     */
    public void setSOTransItems(Object value) {
        this.sOTransItems = value;
    }

    /**
     * Sets currency
     * 
     * @param value Value to set
     */
    public void setCurrency(Object value) {
        this.currency = value;
    }

    /**
     * Sets documentNo
     * 
     * @param value Value to set
     */
    public void setDocumentNo(Object value) {
        this.documentNo = value;
    }

    /**
     * Sets message
     * 
     * @param value Value to set
     */
    public void setMessage(Object value) {
        this.message = value;
    }

    /**
     * Sets transactionType
     * 
     * @param value Value to set
     */
    public void setTransactionType(Object value) {
        this.transactionType = value;
    }

    /**
     * Sets customerId
     * 
     * @param value Value to set
     */
    public void setCustomerId(Object value) {
        this.customerId = value;
    }

    /**
     * Sets vsoePriceList
     * 
     * @param value Value to set
     */
    public void setVsoePriceList(Object value) {
        this.vsoePriceList = value;
    }

    /**
     * Sets dateCreated
     * 
     * @param value Value to set
     */
    public void setDateCreated(Object value) {
        this.dateCreated = value;
    }

    /**
     * Sets exchType
     * 
     * @param value Value to set
     */
    public void setExchType(Object value) {
        this.exchType = value;
    }

    /**
     * Sets createdFrom
     * 
     * @param value Value to set
     */
    public void setCreatedFrom(Object value) {
        this.createdFrom = value;
    }

    /**
     * Sets referenceNo
     * 
     * @param value Value to set
     */
    public void setReferenceNo(Object value) {
        this.referenceNo = value;
    }

    /**
     * Sets billToContactType
     * 
     * @param value Value to set
     */
    public void setBillToContactType(Object value) {
        this.billToContactType = value;
    }

    /**
     * Get all superclasses and interfaces recursively.
     * 
     * @param classes List of classes to which to add all found super classes and interfaces.
     * @param clazz   The class to start the search with.
     */
    private void computeClassHierarchy(Class clazz, List classes) {
        for (Class current = clazz; (current!= null); current = current.getSuperclass()) {
            if (classes.contains(current)) {
                return ;
            }
            classes.add(current);
            for (Class currentInterface: current.getInterfaces()) {
                computeClassHierarchy(currentInterface, classes);
            }
        }
    }

    /**
     * Checks whether the specified class parameter is an instance of {@link List }
     * 
     * @param clazz <code>Class</code> to check.
     * @return
     */
    private boolean isListClass(Class clazz) {
        List<Class> classes = new ArrayList<Class>();
        computeClassHierarchy(clazz, classes);
        return classes.contains(List.class);
    }

    /**
     * Checks whether the specified class parameter is an instance of {@link Map }
     * 
     * @param clazz <code>Class</code> to check.
     * @return
     */
    private boolean isMapClass(Class clazz) {
        List<Class> classes = new ArrayList<Class>();
        computeClassHierarchy(clazz, classes);
        return classes.contains(Map.class);
    }

    private boolean isList(Type type) {
        if ((type instanceof Class)&&isListClass(((Class) type))) {
            return true;
        }
        if (type instanceof ParameterizedType) {
            return isList(((ParameterizedType) type).getRawType());
        }
        if (type instanceof WildcardType) {
            Type[] upperBounds = ((WildcardType) type).getUpperBounds();
            return ((upperBounds.length!= 0)&&isList(upperBounds[ 0 ]));
        }
        return false;
    }

    private boolean isMap(Type type) {
        if ((type instanceof Class)&&isMapClass(((Class) type))) {
            return true;
        }
        if (type instanceof ParameterizedType) {
            return isMap(((ParameterizedType) type).getRawType());
        }
        if (type instanceof WildcardType) {
            Type[] upperBounds = ((WildcardType) type).getUpperBounds();
            return ((upperBounds.length!= 0)&&isMap(upperBounds[ 0 ]));
        }
        return false;
    }

    private boolean isAssignableFrom(Type expectedType, Class clazz) {
        if (expectedType instanceof Class) {
            if (((Class) expectedType).isPrimitive()) {
                if (((Class) expectedType).getName().equals("boolean")&&(clazz == Boolean.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("byte")&&(clazz == Byte.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("short")&&(clazz == Short.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("char")&&(clazz == Character.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("int")&&(clazz == Integer.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("float")&&(clazz == Float.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("long")&&(clazz == Long.class)) {
                    return true;
                }
                if (((Class) expectedType).getName().equals("double")&&(clazz == Double.class)) {
                    return true;
                }
                return false;
            } else {
                return ((Class) expectedType).isAssignableFrom(clazz);
            }
        }
        if (expectedType instanceof ParameterizedType) {
            return isAssignableFrom(((ParameterizedType) expectedType).getRawType(), clazz);
        }
        if (expectedType instanceof WildcardType) {
            Type[] upperBounds = ((WildcardType) expectedType).getUpperBounds();
            if (upperBounds.length!= 0) {
                return isAssignableFrom(upperBounds[ 0 ], clazz);
            }
        }
        return false;
    }

    private Object evaluate(MuleMessage muleMessage, Object source) {
        if (source instanceof String) {
            String stringSource = ((String) source);
            if (stringSource.startsWith(patternInfo.getPrefix())&&stringSource.endsWith(patternInfo.getSuffix())) {
                return expressionManager.evaluate(stringSource, muleMessage);
            } else {
                return expressionManager.parse(stringSource, muleMessage);
            }
        }
        return source;
    }

    private Object evaluateAndTransform(MuleMessage muleMessage, Type expectedType, String expectedMimeType, Object source)
        throws TransformerException
    {
        if (source == null) {
            return source;
        }
        Object target = null;
        if (isList(source.getClass())) {
            if (isList(expectedType)) {
                List newList = new ArrayList();
                Type valueType = ((ParameterizedType) expectedType).getActualTypeArguments()[ 0 ];
                ListIterator iterator = ((List) source).listIterator();
                while (iterator.hasNext()) {
                    Object subTarget = iterator.next();
                    newList.add(evaluateAndTransform(muleMessage, valueType, expectedMimeType, subTarget));
                }
                target = newList;
            } else {
                target = source;
            }
        } else {
            if (isMap(source.getClass())) {
                if (isMap(expectedType)) {
                    Type keyType = Object.class;
                    Type valueType = Object.class;
                    if (expectedType instanceof ParameterizedType) {
                        keyType = ((ParameterizedType) expectedType).getActualTypeArguments()[ 0 ];
                        valueType = ((ParameterizedType) expectedType).getActualTypeArguments()[ 1 ];
                    }
                    Map map = ((Map) source);
                    Map newMap = new HashMap();
                    for (Object entryObj: map.entrySet()) {
                        {
                            Map.Entry entry = ((Map.Entry) entryObj);
                            Object newKey = evaluateAndTransform(muleMessage, keyType, expectedMimeType, entry.getKey());
                            Object newValue = evaluateAndTransform(muleMessage, valueType, expectedMimeType, entry.getValue());
                            newMap.put(newKey, newValue);
                        }
                    }
                    target = newMap;
                } else {
                    target = source;
                }
            } else {
                target = evaluate(muleMessage, source);
            }
        }
        if ((target!= null)&&(!isAssignableFrom(expectedType, target.getClass()))) {
            DataType sourceDataType = DataTypeFactory.create(target.getClass());
            DataType targetDataType = null;
            if (expectedMimeType!= null) {
                targetDataType = DataTypeFactory.create(((Class) expectedType), expectedMimeType);
            } else {
                targetDataType = DataTypeFactory.create(((Class) expectedType));
            }
            Transformer t = muleContext.getRegistry().lookupTransformer(sourceDataType, targetDataType);
            return t.transform(target);
        } else {
            return target;
        }
    }

    /**
     * Invokes the MessageProcessor.
     * 
     * @param event MuleEvent to be processed
     * @throws MuleException
     */
    public MuleEvent process(MuleEvent event)
        throws MuleException
    {
        MuleMessage _muleMessage = event.getMessage();
        IntacctCloudConnectorLifecycleAdapter _castedModuleObject = null;
        if (moduleObject instanceof String) {
            _castedModuleObject = ((IntacctCloudConnectorLifecycleAdapter) muleContext.getRegistry().lookupObject(((String) moduleObject)));
            if (_castedModuleObject == null) {
                throw new MessagingException(CoreMessages.failedToCreate("createSOTransaction"), event, new RuntimeException("Cannot find the configuration specified by the config-ref attribute."));
            }
        } else {
            _castedModuleObject = ((IntacctCloudConnectorLifecycleAdapter) moduleObject);
        }
        try {
            String _transformedFunctionControlId = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_functionControlIdType").getGenericType(), null, functionControlId));
            String _transformedTransactionType = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_transactionTypeType").getGenericType(), null, transactionType));
            Map<String, Object> _transformedDateCreated = ((Map<String, Object> ) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_dateCreatedType").getGenericType(), null, dateCreated));
            String _transformedCreatedFrom = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_createdFromType").getGenericType(), null, createdFrom));
            String _transformedCustomerId = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_customerIdType").getGenericType(), null, customerId));
            String _transformedDocumentNo = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_documentNoType").getGenericType(), null, documentNo));
            String _transformedReferenceNo = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_referenceNoType").getGenericType(), null, referenceNo));
            String _transformedTermName = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_termNameType").getGenericType(), null, termName));
            Map<String, Object> _transformedDateDue = ((Map<String, Object> ) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_dateDueType").getGenericType(), null, dateDue));
            String _transformedMessage = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_messageType").getGenericType(), null, message));
            String _transformedShippingMethod = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_shippingMethodType").getGenericType(), null, shippingMethod));
            org.mule.module.intacct.ContactType _transformedBillToContactType = ((org.mule.module.intacct.ContactType) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_billToContactTypeType").getGenericType(), null, billToContactType));
            List<Map<String, Object>> _transformedBillToContacts = ((List<Map<String, Object>> ) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_billToContactsType").getGenericType(), null, billToContacts));
            org.mule.module.intacct.ContactType _transformedShipToContactType = ((org.mule.module.intacct.ContactType) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_shipToContactTypeType").getGenericType(), null, shipToContactType));
            List<Map<String, Object>> _transformedShipToContacts = ((List<Map<String, Object>> ) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_shipToContactsType").getGenericType(), null, shipToContacts));
            String _transformedExternalId = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_externalIdType").getGenericType(), null, externalId));
            String _transformedBasecurr = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_basecurrType").getGenericType(), null, basecurr));
            String _transformedCurrency = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_currencyType").getGenericType(), null, currency));
            org.mule.module.intacct.ExchType _transformedExchType = ((org.mule.module.intacct.ExchType) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_exchTypeType").getGenericType(), null, exchType));
            List<Map<String, Object>> _transformedExchRateDatesOrExchRateTypesOrExchRates = ((List<Map<String, Object>> ) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_exchRateDatesOrExchRateTypesOrExchRatesType").getGenericType(), null, exchRateDatesOrExchRateTypesOrExchRates));
            String _transformedVsoePriceList = ((String) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_vsoePriceListType").getGenericType(), null, vsoePriceList));
            List<Map<String, Object>> _transformedCustomFields = ((List<Map<String, Object>> ) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_customFieldsType").getGenericType(), null, customFields));
            List<Map<String, Object>> _transformedSOTransItems = ((List<Map<String, Object>> ) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_sOTransItemsType").getGenericType(), null, sOTransItems));
            List<Map<String, Object>> _transformedSubTotals = ((List<Map<String, Object>> ) evaluateAndTransform(_muleMessage, CreateSOTransactionMessageProcessor.class.getDeclaredField("_subTotalsType").getGenericType(), null, subTotals));
            retryCount.getAndIncrement();
            Object resultPayload;
            resultPayload = _castedModuleObject.createSOTransaction(_transformedFunctionControlId, _transformedTransactionType, _transformedDateCreated, _transformedCreatedFrom, _transformedCustomerId, _transformedDocumentNo, _transformedReferenceNo, _transformedTermName, _transformedDateDue, _transformedMessage, _transformedShippingMethod, _transformedBillToContactType, _transformedBillToContacts, _transformedShipToContactType, _transformedShipToContacts, _transformedExternalId, _transformedBasecurr, _transformedCurrency, _transformedExchType, _transformedExchRateDatesOrExchRateTypesOrExchRates, _transformedVsoePriceList, _transformedCustomFields, _transformedSOTransItems, _transformedSubTotals);
            TransformerTemplate.OverwitePayloadCallback overwritePayloadCallback = null;
            if (resultPayload == null) {
                overwritePayloadCallback = new TransformerTemplate.OverwitePayloadCallback(NullPayload.getInstance());
            } else {
                overwritePayloadCallback = new TransformerTemplate.OverwitePayloadCallback(resultPayload);
            }
            List<Transformer> transformerList;
            transformerList = new ArrayList<Transformer>();
            transformerList.add(new TransformerTemplate(overwritePayloadCallback));
            event.getMessage().applyTransformers(event, transformerList);
            retryCount.set(0);
            return event;
        } catch (Exception e) {
            throw new MessagingException(CoreMessages.failedToInvoke("createSOTransaction"), event, e);
        }
    }

}
